#!/usr/bin/env python3
"""
Recupera a partir de mi BD de correos las notificaciones que MelodySoft me enviaba, y con ellas reconstruye los
mensajes de mi libro de visitas (no todos pues usaba otros correos antes de GMail).
Qudará en https://www.danielclemente.com/libro/archivo.html

2022, Daniel Clemente.

Código en dominio público.
Este código es muy guarro; viene de muchas pruebas chapuceras; lo hice para usarlo sólo yo, una sola vez, y
no me importa que no se entienda o que no sea reusable.
Se usan funciones mías que no he publicado. Tampoco he publicado mi infraestructura de correos
(ej. la BD sqlite donde los indexo). Algún día…
"""

import imp
import os
import sqlite3

# No me va un „import funcompartidas“ etc. pues no estoy en ~/ut/ y no uso $PYTHONPATH
# Hago la típica guarrada. Hay formas mejores…
# Algunas cosas importadas no las uso aquí pero son necesarias para hacer los otros imports que sí uso.
funcompartidas = imp.load_source('funcompartidas', os.path.expanduser('~/ut/funcompartidas.py'))
filtro_borra_palabras_comunes = imp.load_source('filtro_borra_palabras_comunes', os.path.expanduser('~/ut/filtro_borra_palabras_comunes.py'))
datos_de_proyecto = imp.load_source('datos_de_proyecto', os.path.expanduser('~/ut/datos_de_proyecto.py'))
grafloquesea = imp.load_source('grafloquesea', os.path.expanduser('~/ut/grafloquesea.py'))
gmvault_indexado = imp.load_source('gmvault_indexado', os.path.expanduser('~/ut/gmvault_indexado.py'))

from gmvault_indexado import RUTA_BD_SQLITE
from gmvault_indexado import dict_factory
import email
import gzip
import re
import datetime
from dateutil.parser import isoparse

# FISAL = "/home/dc/n/pruebarecucom.html"
FISAL = "/home/dc/repoweb/libro/archivo.html"
BUSCA_CAMPOS = (
  "^Usuario:.+\n"
  "Libro:.+\n"
  "Autor:\n(?P<autor>.*?)\n\n"
  "E-Mail:.*?\n"
  "(Link:\n(?P<enlace1>.*?)\n\n)?"
  "(País:\n(?P<país>.*?)\n\n)?"
  "(Edad:\n(?P<edad>.*?)\n\n)?"
  "(Link:\n(?P<enlace2>.*?)\n\n)?"
  "(Nos conociste por:\n(?P<noscon>.*?)\n\n)?"
  "(Opinión:\n(?P<opinión>.+?)\n\n)?"
  "(Link:\n(?P<enlace3>.*?)\n\n)?"
  "Mensaje:\n(?P<mensaje>.+)\n\n"
  "\\*\\*\\* http://gbooks1.melodysoft.com/app\?ID=WebDCL\n$"
)

DEPURANDO = True
DEPURANDO = False

if 1:
    conn = sqlite3.connect(RUTA_BD_SQLITE)
    conn.row_factory = dict_factory
    c = conn.cursor()
    mensajes = c.execute("""SELECT * from correos where asunto LIKE "Libro WebDCL%" ORDER BY fecha;""")
    mensajes = list(mensajes)
    print(len(mensajes))
    print(mensajes[0])
    sal = open(FISAL, "w")
    # No tengo ganas de usar jinja ahora
    sal.write("""
<!doctype html>
<html lang="en">
  <head>
    <meta charset="UTF-8"/>
    <!-- Este fichero se autogenera mediante este script -->
    <meta name="generator" content="https://www.danielclemente.com/scripts/recupera_libro_visitas.py" />
    <!-- Considero estos comentarios externos a mi página, y con información de otros, y no creo que tengan que ser preservados por todos. Si tú (persona) quieres guardar una copia, adelante. Pero en principio, que no se indexen  -->
    <meta name="robots" content="noindex,follow">
    <title>Archivo de comentarios 2005-2020, rescatados</title>
    <style>
      pre{white-space: pre-wrap;}
      body{margin:3%;}
    </style>
  </head>
  <body>

<p>Tenía un <a href="https://www.danielclemente.com/libro/">libro de visitas a mi web</a> en un servicio externo, y en 2021 cerró y se perdieron. He recuperado la mayoría de comentarios a partir de los avisos que me mandaban a mi correo.</p>
<p>Preservo aquí los comentarios, porque la gente pasó un tiempo escribiéndolos y no quiero perderlos otra vez. <strong>No representan mi opinión sino la de otros</strong> y de hace tiempo.</p>
<p>Se incluyen comentarios desagradables e inmaduros; no los censuro (sólo quité el espam automatizado en inglés). Casi todo es del período cercano a 2004; yo era adolescente y atacando a Internet Explorer (por tanto concuerdo con las críticas a mi prepotencia. En 2022 he mejorado pero no he cambiado rumbo. Por cierto, al final entre todos (<a href="https://news.ycombinator.com/item?id=19798678">ej. de otros</a>) <em>ganamos</em> la batalla contra IE 6 (<a href="https://en.wikipedia.org/w/index.php?title=Internet_Explorer&oldid=1082454503#Market_adoption_and_usage_share">resultados</a>), y valió la pena.</p>
<p>También mantengo esta sección para leerla de vez en cuando y recordarme que, a pesar de mis desgracias y bloqueos, a veces he conseguido hacer mi objetivo: motivar a otros a fomentar el aprendizaje y la ética (en lo segundo incluyo lo del software libre). También veo que muchas personas están contentas de que yo comparta conocimientos en una web personal. ¿Has pensado tú en compartir los tuyos en tu web?</p>
<p>Por ahora (2022) sigo sin <a href="https://www.danielclemente.com/libro/">libro de visitas</a>; ahí he puesto las ideas para publicar comentarios o discutir.</p>

<!-- Nota para mí: en 2022 me he hecho un resumen del feedback interesante de ~2005-2016, ver notas en MiWeb.org -->
<!-- Nota para ti: si quieres añadir feedback, mándamelo al correo y dime que es para publicarlo aquí. Y dime también cómo mejorar este sistema (ve https://www.danielclemente.com/libro/ ) -->
<hr />
""")
    for m in mensajes:
        co = gzip.open("/home/dc/.retpoŝto/gmvault-db/db/%s/%s" % (m['año_mes'], m['fichero_eml']), "rb")
        em = email.message_from_binary_file(co)
        fecha = m['fecha']
        # from rfoo.utils import rconsole; rconsole.spawn_server()
        # import time; time.sleep(300000)
        assert em.get_content_type() == "text/plain"
        body = em.get_payload(decode=True)
        # texto = str(body)
        texto = body.decode("utf-8")

        # ¿estos dos son errores míos? Otros campos como „Dirección IP“ salen bien
        texto = texto.replace("Paï¿½s:", "País:")
        texto = texto.replace("Opiniï¿½n:", "Opinión:")

        # y más:
        # assert '﨎' not in texto # 﨎
        if 0 and 'gedesp' in texto:
            import sys; sys.stdout = sys.__stdout__; import ipdb; ipdb.set_trace(); "Prueba también: ipy";
        # texto = texto.replace("Pa﨎:", "País:")
        # texto = texto.replace("Opini:", "Opinión:")
        texto = texto.replace("Pa&#64014;:", "País:")
        texto = texto.replace("Opini&#57954;:", "Opinión:")

        # más, esta vez no haz otros campos
        texto = texto.replace("Opini�n:", "Opinión:")

        # No sé cómo esta corrupción llegó al campo e-mail de 1 persona…
        texto = texto.replace("@Âyahoo.com", "@yahoo.com")

        if 'Opini?n:\n' in texto or 'OpiniÃ³n' in texto:
            # ¡espam! Cientos. Por suerte están mal codificados
            continue
        elif 'Edad:\nUnknown' in texto:
            # ¡espam! Muchos
            continue
        elif '[url=' in texto or '[/url' in texto:
            # ¡espam! Muchos
            continue
        elif re.search("\\b(pantie|escorts|this site|il vostro|83.16.196.82|91.232.96.4|67.18.98.36|66.232.123.181|72.232.38.10|203.162.27.95|79.135.167.26|195.225.177.34|195.225.176.13|65.110.43.32|65.110.43.30|201.236.233.14|RcpabOWX|EKwQPvWGRbfj|9bn1vm5xxh5|u4l91ffk57|jy57qkkbx1|nyko9m7qd3|fyckdlxx|FKLdzmwRARUh|indyclVyUCXeqXpYG|nfpdkwx8pybwp|constructores.web|Paintball Madrid|magnetoterapia|kalendarze|emmakevin|good |interesting|higuys|Well done|Nice site!?|Lovely site|nice page|great site|great work|money |poker|online-casino|casino-online|casino-1|ny054|gambling-7|comics768mb|videos675mb|fat451mb|cams56teen|download234mb|cams567teen|select3425mb|cams234teen|mature256mb|cams963teen|famous463mb|xopopo|paydayloansonline|Bibinga|vacationonmars|emeraldring|universities|juicypornhost|pornzonehost|sexpornhost|deadzoom|hidden cam|pochta|acetaminophen|buratin|phendimetrazine|verapamil|pills/meridia|nortriptyline|hydrocodone|allopurinol|codeine)\\b", texto, re.IGNORECASE):
            # lo detecto como espam. Específico en mi web
            continue
        elif re.search("E-Mail:\\n.*\.pl\\n", texto):
            # mucho polaco
            continue
        campos = re.match(BUSCA_CAMPOS, texto, re.DOTALL)
        # assert campos
        if 1 and 'MENSAJE PRIVADO - NO SE INCLUIRÁ EN LA SECCIÓN FIRMAS' in texto:
            # saltarse los privados
            # continue
            texto_reformado = "(Mensaje privado) (el enviante lo mandó sólo a mi correo)"
        elif not campos:
            texto_reformado = """No encontrados campos en mensaje: «%s»\n""" % texto
            raise NotImplementedError("Ya conseguí arreglar esto; no debería fallar más")
        else:
            # grupos_vistos = campos.groups()
            # print(grupos_vistos)
            texto_reformado = """
Autor: {autor}

País: {país}

Edad: {edad}

Nos conociste por: {noscon}

Opinión: {opinión}

Link: {link}

Mensaje: {mensaje}
""".format(
    autor=campos.group('autor'),
    país=campos.group('país') or "",
    edad=campos.group('edad') or "",
    noscon=campos.group('noscon') or "",
    opinión=campos.group('opinión') or "",
    link=campos.group('enlace1') or campos.group('enlace2') or campos.group('enlace3') or "",
    mensaje=campos.group('mensaje'),
)
            # etc. etc.
            pass  # texto = 
        if 1:
            assert "Opiniï¿½n:" not in texto_reformado
            assert "Opini�n:" not in texto_reformado
            assert "@Â" not in texto, texto  # ¿o alguien lo puso aposta? Muy extraño

        texto_reformado = texto_reformado.replace("<", "&lt;")
        texto_reformado = texto_reformado.replace(">", "&gt;")
        sal.write("""\n<dt>%s</dt>\n<dd><pre>%s</pre></dd>\n""" % (isoparse(fecha).strftime("%Y-%m-%d %a %H:%M"), texto_reformado))
        if DEPURANDO:
            sal.write("""\n<dt>mens. original del anterior</dt>\n<dd><pre style="font-size: 80%%; color: gray;">%s</pre></dd>\n""" % (texto.replace("<", "&lt;").replace(">", "&gt;")))
    sal.write("""

  </body>
</html>
""")
    sal.close()
    c.close()
    print("Escrito a %s" % FISAL)
